import javax.swing.*;
import java.awt.event.*;
import java.awt.*;
import java.util.*;
import javax.swing.table.*;
import javax.swing.border.*;
import java.io.*;
import java.util.regex.*;


/**
* GridBagLayout Editor is an application that aims to simplify creation of GridBagLaoyout based application
* GUI interfaces. GBLEditor is the master-class of this application. This class creates the main window, and all
* GUI components, and handles all data-flow between user/application and between objects.
*/
public class GBLEditor extends JFrame {
	int rowcount = 0;
	GBLModel gblm = new GBLModel();
	JTable table = new JTable(gblm);
	
	GBLEditorToolbar toolbar = null;
	GBLEditorMenu menu = null;
	
	/**
	* The constructor takes no arguments. Though a main function exists, the meat and bone of the
	* application is actually executed in the constructor of GBLEditor.
	*/
	public GBLEditor () {
		super(GBLText.t_title);

		JScrollPane tablescroller = new JScrollPane(table);
		JComboBox typeSelect = new JComboBox (GBLRow.getTypes());
		TableColumn typeColumn = table.getColumnModel().getColumn (0);
		typeColumn.setCellEditor (new DefaultCellEditor(typeSelect));


		JComboBox fillSelect = new JComboBox (GBLRow.getFillTypes());
		fillSelect.setRenderer (new DropDownRenderer ());
    		TableColumn fillColumn = table.getColumnModel().getColumn (7);
		fillColumn.setCellEditor (new DefaultCellEditor(fillSelect));
		fillColumn.setCellRenderer (new CellTypeRenderer());
				
		JComboBox anchorSelect = new JComboBox (GBLRow.getAnchorTypes());
		anchorSelect.setRenderer (new DropDownRenderer ());
		TableColumn anchorColumn = table.getColumnModel().getColumn (8);
		anchorColumn.setCellEditor (new DefaultCellEditor(anchorSelect));
		anchorColumn.setCellRenderer (new CellTypeRenderer());
		
		
		table.getColumnModel().getColumn(0).setPreferredWidth(100);
		table.getColumnModel().getColumn(1).setPreferredWidth(110);
		table.getColumnModel().getColumn(2).setPreferredWidth(130);
		table.getColumnModel().getColumn(3).setPreferredWidth(60);
		table.getColumnModel().getColumn(4).setPreferredWidth(60);
		table.getColumnModel().getColumn(5).setPreferredWidth(60);
		table.getColumnModel().getColumn(6).setPreferredWidth(60);
		table.getColumnModel().getColumn(7).setPreferredWidth(150);
		table.getColumnModel().getColumn(8).setPreferredWidth(150);
	
		toolbar = new GBLEditorToolbar();
		menu = new GBLEditorMenu();
		/**
		* inner anonumous actionlistener class, that simply creates a new aboutbox...
		*/
		toolbar.b_help.addActionListener(new ActionListener () {
			public void actionPerformed (ActionEvent ae) {
				AboutBox about = new AboutBox(); 
			}
		});
		
		toolbar.b_new.addActionListener(new NewClearAction ());
		toolbar.b_add.addActionListener(new AddRowAction ());
		toolbar.b_up.addActionListener(new MoveRowUpAction());
		toolbar.b_down.addActionListener(new MoveRowDownAction());
		
		toolbar.b_export.addActionListener(new GenerateCodeAction());

		menu.mi_about.addActionListener(new ActionListener () {
			public void actionPerformed (ActionEvent ae) {
				AboutBox about = new AboutBox(); 
			}
		});
		menu.mi_new.addActionListener(new NewClearAction ());
		menu.mi_quit.addActionListener(new Exit());
		menu.mi_addrow.addActionListener(new AddRowAction());
		menu.mi_moveup.addActionListener(new MoveRowUpAction());
		menu.mi_movedn.addActionListener(new MoveRowDownAction());
		
		getContentPane().setLayout(new BorderLayout());
		getContentPane().add(toolbar, BorderLayout.NORTH);
		getContentPane().add(tablescroller, BorderLayout.CENTER);

		setJMenuBar(menu);
		setSize(880, 350);
		setVisible(true);
		setDefaultCloseOperation(EXIT_ON_CLOSE); // Burde sikkert bruke Exit klassen?

	}

	/**
	* Main method of the GridBagLayout Editor. Creates a new instance of GBLEditor, and
	* by doing so, hands execution over to the constructor of GBLEditor, and to the 
	* GUI threads of Java...
	*/
	public static void main (String arg[]) {
		GBLEditor editor = new GBLEditor();
	}
	
	/**
	* ActionListener that is fired whenever an action is performed that ought to result in a
	* new row in the table..
	*/
	protected class AddRowAction implements ActionListener {
		public void actionPerformed (ActionEvent ae) {
			gblm.addRow(new GBLRow(rowcount++)); 
		}
	}
	/**
	* ActionListener that is fired whenever an action is performed that ought to result in a
	* row beeing moved up in the table..
	*/
	protected class MoveRowUpAction implements ActionListener {
		public void actionPerformed (ActionEvent ae) {
			if ( table.getSelectedRow() > 0 ) {
				int selectedrow = table.getSelectedRow();
				GBLRow temp1 = (GBLRow)gblm.data.get(selectedrow);
				GBLRow temp2 = (GBLRow)gblm.data.get(selectedrow-1);
				gblm.data.set(selectedrow-1, temp1);
				gblm.data.set(selectedrow, temp2);
				table.setRowSelectionInterval(selectedrow-1,selectedrow-1);
			}
		}
	}
	
	protected class NewClearAction implements ActionListener {
		public void actionPerformed (ActionEvent ae) {
			gblm.data.clear();
			//table.setRowSelectionInterval(0,0);
			table.repaint();
			table.addNotify();
			rowcount = 0;
		}
	}
	
	/**
	* ActionListener that is fired whenever an action is performed that ought to result in a
	* row beeing moved down in the table..
	*/
	protected class MoveRowDownAction implements ActionListener {
		public void actionPerformed (ActionEvent ae) {
			if ((table.getSelectedRow() < table.getRowCount()-1) && (table.getSelectedRow() > -1)) {
				int selectedrow = table.getSelectedRow();
				GBLRow temp1 = (GBLRow)gblm.data.get(selectedrow);
				GBLRow temp2 = (GBLRow)gblm.data.get(selectedrow+1);
				gblm.data.set(selectedrow+1, temp1);
				gblm.data.set(selectedrow, temp2);
				table.setRowSelectionInterval(selectedrow+1,selectedrow+1);
			}
		}
	}
	protected class GenerateCodeAction implements ActionListener {
		public void actionPerformed (ActionEvent ae) {
			
			File file = null;
			JFileChooser fc = new JFileChooser();
			
			// Start in current directory
			fc.setCurrentDirectory(new File("."));
			
			// Set filter for Java source files.
			//fc.setFileFilter(javaFilter);
			
			// Set to a default name for save.
			fc.setSelectedFile(file);
			
			// Open chooser dialog
			//int result = fc.showSaveDialog(null);
			int result = fc.showSaveDialog(GBLEditor.this);
			
			if( result == JFileChooser.CANCEL_OPTION) {
				// CANCEL PRESSED!!
				//System.out.println("User cancelled saving");
				return;
			} else if ( result == JFileChooser.APPROVE_OPTION) {
				file = fc.getSelectedFile();
				if( file.exists()) {
					int response = JOptionPane.showConfirmDialog(null,
						"Overwrite existing file?",
						"Confirm Overwrite",
						JOptionPane.OK_CANCEL_OPTION,
						JOptionPane.QUESTION_MESSAGE);
					if( response == JOptionPane.CANCEL_OPTION) {
						// FILE EXISTS, USER CANCELLED
						//System.out.println("User cancelled overwrite");
						return;
					}
				}
				// WRITE TO FILE
				//System.out.println("Write to file");
				//System.out.println("Name:        " + file.getName());
				//System.out.println("Path:        " + file.getPath());
				String classname = file.getName().split("\\.java$")[0];
				//System.out.println("Class name:  " + classname);
				//System.out.println("HEI!");
				
				try {
					// OBSOBSOBS! FileWriter SKAL byttes til en BufferedWriter, slik at newLine() kan benyttes
					FileWriter fw = new FileWriter(file);
					fw.write("import java.awt.*;\n");
					fw.write("import javax.swing.*;\n");
					fw.write("\n");
					fw.write("public class " + classname + " extends JPanel {\n");
					Object[] rows = gblm.getRowArray();
					//System.out.println("Array length: " + rows.length);
					for (int i = 0; i < rows.length; i++) {
						GBLRow row = (GBLRow)rows[i];
						fw.write("    ");
						if (row.getType().equals("JComboBox")) {
							fw.write("String ");
							fw.write(row.getName());
							fw.write("Data = \"");
							fw.write(row.getText());
							fw.write("\";\n");
							fw.write("    DefaultComboBoxModel ");
							fw.write(row.getName());
							fw.write("Model = new DefaultComboBoxModel (");
							fw.write(row.getName());
							fw.write("Data.split (\"[\\\\p{Punct}\\\\s]+\"));\n");
							fw.write("    JComboBox ");
							fw.write(row.getName());
							fw.write(" = new JComboBox (");
							fw.write(row.getName());
							fw.write("Model);\n");
  						} else if (row.getType().equals("JList")) {
							fw.write("String ");
							fw.write(row.getName());
							fw.write("Data = \"");
							fw.write(row.getText());
							fw.write("\";\n");
							fw.write("    DefaultComboBoxModel ");
							fw.write(row.getName());
							fw.write("Model = new DefaultComboBoxModel (");
							fw.write(row.getName());
							fw.write("Data.split (\"[\\\\p{Punct}\\\\s]+\"));\n");
							fw.write("    JList ");
							fw.write(row.getName());
							fw.write(" = new JList (");
							fw.write(row.getName());
							fw.write("Model);\n");
						} else if (row.getType().equals("JSpinnerList")) {
							fw.write("String  ");
							fw.write(row.getName());
							fw.write("Data = \"");
							fw.write(row.getText());
							fw.write("\";\n");
							fw.write("    SpinnerListModel ");
							fw.write(row.getName());
							fw.write("Model = new SpinnerListModel (");
							fw.write(row.getName());
							fw.write("Data.split (\"[\\\\p{Punct}\\\\s]+\"));\n");
							fw.write("    JSpinner ");
							fw.write(row.getName());
							fw.write(" = new JSpinner (");
							fw.write(row.getName());
							fw.write("Model);\n");
						} else if (row.getType().equals("JSpinnerNumber")) {
							fw.write("JSpinner " + row.getName());
							fw.write(" = new JSpinner(new SpinnerNumberModel(1.0, 1.0, 5.0, 1.0));\n");
						} else {
							fw.write(row.getType() + " " + row.getName());
							fw.write(" = new " + row.getType() + "(\"" + row.getText() + "\");\n");
						}
					}
					fw.write("\n    ");
					fw.write("public " + classname + "() {\n");
					fw.write("        ");
					fw.write("GridBagLayout layout = new GridBagLayout ();\n");
					fw.write("        ");
					fw.write("GridBagConstraints gbc = new GridBagConstraints();\n");
					fw.write("        ");
					fw.write("setLayout (layout);\n");
					fw.write("\n");
					
					for (int i = 0; i < rows.length; i++) {
						GBLRow row = (GBLRow)rows[i];
						
						fw.write("        gbc.gridx = " + row.getColumn() + ";\n");
						fw.write("        gbc.gridy = " + row.getRow() + ";\n");
						fw.write("        gbc.gridwidth = " + row.getColumns() + ";\n");
						fw.write("        gbc.gridheight = " + row.getRows() +";\n");
						fw.write("        gbc.anchor = java.awt.GridBagConstraints." + row.getAnchor() + ";\n");
						fw.write("        gbc.fill = java.awt.GridBagConstraints." + row.getFill() + ";\n");
						
						if (row.getType().equals("JTextArea")) {
							fw.write("        " + row.getName() + ".setLineWrap (true);\n");
							fw.write("        " + row.getName() + ".setWrapStyleWord (true);\n");
							fw.write("        JScrollPane " + row.getName() + "ScrollPane =");
							fw.write(" new JScrollPane (" + row.getName() +");\n");
							fw.write("        layout.setConstraints (" + row.getName() + "ScrollPane, gbc);\n");
							fw.write("        add (" + row.getName() + "ScrollPane);\n");
						} else if (row.getType().equals("JList")) {
							fw.write("        JScrollPane " + row.getName() + "ScrollPane =");
							fw.write(" new JScrollPane (" + row.getName() +");\n");
							fw.write("        layout.setConstraints (" + row.getName() + "ScrollPane, gbc);\n");
							fw.write("        add (" + row.getName() + "ScrollPane);\n");
						} else {
							fw.write("        layout.setConstraints (" + row.getName() + ", gbc);\n");
							fw.write("        add (" + row.getName() + ");\n");
						}
						fw.write("\n");
					}
					
					// RESTEN AV KODEN HER!
					fw.write("    ");
					fw.write("}\n");
					fw.write("}\n");
					fw.close();
				} catch (Exception e) {
					System.out.println("Exception!!!");
					e.printStackTrace();
				}
			} else {
				System.out.println("Whooot?");
				return;
			}
		}
	}

}
