/**
* This is the actual data object that GBLEditor is centered around.
*/
public class GBLRow {
	String Type = null;
//	int Type;
	String VarName = null;
	String FieldText = null;
	int Row;
	int Column;
	int Rows;
	int Columns;
//	int Fill;
//	int Anchor;
	String Fill = null;
	String Anchor = null;
	
	private static final String FillTypes[] = {"NONE","CENTER","HORIZONTAL","VERTICAL","BOTH"};
	private static final String AnchorTypes[] = { "CENTER", "NORTH", "NORTHEAST", "EAST", "SOUTHEAST", "SOUTH", "SOUTHWEST", "WEST", "NORTHWEST"};
	private static final String TypeTypes[] = {"JLabel", "JButton", "JTextField", "JTextArea", "JCheckBox", "JList", "JComboBox", "JSpinnerList", "JSpinnerNumber"}; //, "JSpinner"};
	
	public GBLRow (int count) {
		//Type = 0;
		Type = new String(TypeTypes[0]);
		VarName = new String("new" + count);
		FieldText = new String("");
		Row = 1;
		Column = 1;
		Rows = 1;
		Columns = 1;
//		Fill = 0;
//		Anchor = 0;
		Fill = new String(FillTypes[0]);
		Anchor = new String(AnchorTypes[0]);
	}
	/**
	* Returns an array of all selectable GUI component types
	* @return String[]
	*/
	public static String [] getTypes () {
		return TypeTypes;
	}
	
	/**
	* Returns an array containing the selectable types of
	* fill that may be used/selected
	* @return String[]
	*/
	public static String [] getFillTypes () {
		return FillTypes;
	}
	
	/**
	* Returns an array containing the selectable types of
	* grid-bag-layout anchoring that may be used/selected
	* @return String[]
	*/
	public static String [] getAnchorTypes () {
		return AnchorTypes;
	}
	
	/**
	* Returns the integer representation of the GUI component type
	
	*/
	//public int getType () { return Type; }
	public String getType () { return Type; }
	
	/**
	* Returns the name of the GUI component..
	* @return String
	*/
	public String getName () { return VarName; }
	
	/**
	* Returns the default text to be contained in a text-based element
	* @return String
	*/
	public String getText () { return FieldText; }
	
	
	/**
	* Returns the row a component will occupy.
	* @return int
	*/
	public int getRow () { return Row;}
	
	/**
	* Returns the column a component will occupy.
	* @return int
	*/

	public int getColumn () { return Column; }
	
	/**
	* Returns the number of rows a component will occupy.
	* @return int
	*/
	public int getRows () { return Rows; }
	
	/**
	* Returns the number of columns a component will occupy.
	* @return int
	*/
	public int getColumns () { return Columns; }
	
	/**
	* Returns the integer representation of the GBL-anchor
	*/
	//public int getAnchor () { return Anchor; }
	public String getAnchor () { return Anchor; }
	
	/**
	* Returns the integer representation of the GBL-fill
	*/
	//public int getFill () { return Fill; }
	public String getFill () { return Fill; }
		
	/**
	* Sets the java component type of a GBL-component row. The type is selected by an
	* int, representing each of the elements in the array returned by getTypes.
	*/
	//public void setType (int newtype) { Type = newtype; }
	
	/**
	* Sets the java component type of a GBL-component row
	*/
	/*
	public void setType (String type) {
		for (int i=0; i<TypeTypes.length; i++) {
			if (type.equals (TypeTypes[i])) {
				setType (i);
				break;
      			}
		}
	}
	*/
	public void setType (String type) { Type = type; }
	
	/**
	* sets the name of the resulting variable generated.
	*/
	public void setName ( String newname ) { VarName = newname; }
	
	/**
	* Sets the default text that will be applied to a text-based element.
	*/
	public void setText ( String newtext ) { FieldText = newtext; }
	
	
	/**
	* Sets the row.
	*/
	public void setRow ( int newrow ) { Row = newrow; } 
	
	/**
	* Sets the column.
	*/
	public void setColumn ( int newcolumn ) { Column = newcolumn; }
	
	/**
	* Sets the height in number of rows for the resulting GUI-component.
	*/
	public void setRows ( int newrows ) { Rows = newrows; }
	
	/**
	* Sets the width in number of columns for the resulting GUI-component.
	*/
	public void setColumns ( int newcolumns ) { Columns = newcolumns; }
	
	/**
	* Sets the anchoring on a GBL-component row. The anchor is selected by an
	* int, representing each of the elements in the array returned by getAnchorTypes.
	*/
	//public void setAnchor ( int newanchor ) { Anchor = newanchor; }
	public void setAnchor ( String newanchor ) { Anchor = newanchor; }	
	/**
	* Sets the type of fill on a GBL-component row. The fill is selected by an
	* int, representing each of the elements in the array returned by getFillTypes.
	*/
	//public void setFill ( int newfill ) { Fill = newfill; }
	public void setFill ( String newfill ) { Fill = newfill; }
}